﻿/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

//1
using UnityEngine;
using UnityEditor;

//2
public class LevelEditorWindow : EditorWindow {
  //3
  int selectedAction = 0;

  [MenuItem ("Window/Level Editor")]
  static void Init () {
    LevelEditorWindow window = (LevelEditorWindow)EditorWindow.GetWindow (typeof (LevelEditorWindow));
    window.Show();
  }

  void OnGUI() {
    float width = position.width - 5;
    float height = 30;
    string[] actionLabels = new string[] {"X", "Create Spot", "Delete Spot"};
    selectedAction = GUILayout.SelectionGrid (selectedAction, actionLabels, 3, GUILayout.Width (width), GUILayout.Height (height));
  }

  void OnScene(SceneView sceneview) {
    Event e = Event.current;
    if (e.type == EventType.MouseUp) {
      Ray r = Camera.current.ScreenPointToRay (new Vector3 (e.mousePosition.x, Camera.current.pixelHeight -e.mousePosition.y));

      if (selectedAction == 1) {
        RaycastHit2D hitInfo2D = Physics2D.GetRayIntersection (r, Mathf.Infinity, 1 << LayerMask.NameToLayer ("Background"));
        if (hitInfo2D) {
          GameObject spotPrefab = AssetDatabase.LoadAssetAtPath<GameObject> ("Assets/Prefabs/Openspot.prefab");
          GameObject spot = (GameObject)PrefabUtility.InstantiatePrefab (spotPrefab);
          spot.transform.position = hitInfo2D.point;
          Undo.RegisterCreatedObjectUndo (spot, "Create spot");
        }
        Debug.Log("Create OpenSpot");
      } else if (selectedAction == 2) {
        RaycastHit2D hitInfo2D = Physics2D.GetRayIntersection (r, Mathf.Infinity, 1 << LayerMask.NameToLayer ("Spots"));
        if (hitInfo2D) {
          GameObject selectedOpenSpot = hitInfo2D.collider.gameObject;
          Undo.DestroyObjectImmediate (selectedOpenSpot);
        }
        Debug.Log("Delete OpenSpot");
      }
    }
  }

  void OnEnable() {
    SceneView.onSceneGUIDelegate -= OnScene;
    SceneView.onSceneGUIDelegate += OnScene;
  }

}