﻿/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

//1
using UnityEditor;
using UnityEngine;

//2
[CustomPropertyDrawer(typeof(Wave))]
//3
public class WavePropertyDrawer : PropertyDrawer {
  //4
  private const int spriteHeight = 50;

  public override void OnGUI (Rect position, SerializedProperty property, GUIContent label) {
    EditorGUI.PropertyField(position, property, label, true);
    // 1
    if (property.isExpanded) {
      // 2
      SerializedProperty enemyPrefabProperty = property.FindPropertyRelative ("enemyPrefab");
      GameObject enemyPrefab = (GameObject) enemyPrefabProperty.objectReferenceValue;
      // 3
      if (enemyPrefab != null) {
        SpriteRenderer enemySprite = enemyPrefab.GetComponentInChildren<SpriteRenderer> ();
        // 4
        int previousIndentLevel = EditorGUI.indentLevel;
        EditorGUI.indentLevel = 2;
        // 5
        Rect indentedRect = EditorGUI.IndentedRect (position);
        float fieldHeight = base.GetPropertyHeight (property, label) + 2;
        Vector3 enemySize = enemySprite.bounds.size;
        Rect texturePosition = new Rect (indentedRect.x, indentedRect.y + fieldHeight * 4, enemySize.x / enemySize.y * spriteHeight, spriteHeight);
        // 6
        EditorGUI.DropShadowLabel(texturePosition, new GUIContent(enemySprite.sprite.texture));
        // 7
        EditorGUI.indentLevel = previousIndentLevel;
      }
    }
  }

  public override float GetPropertyHeight(SerializedProperty property, GUIContent label) {
    SerializedProperty enemyPrefabProperty = property.FindPropertyRelative ("enemyPrefab");
    GameObject enemyPrefab = (GameObject)enemyPrefabProperty.objectReferenceValue;
    if (property.isExpanded && enemyPrefab != null) {
    return EditorGUI.GetPropertyHeight (property) + spriteHeight; } else {
      return EditorGUI.GetPropertyHeight (property);
    }
  }

}