﻿/*
 * Copyright (c) 2016 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

using UnityEngine;
using System;
using System.Collections;

public class BulletBehavior : MonoBehaviour {

  public BulletConfig bulletConfig;
 [HideInInspector]
 [NonSerialized]
 public GameObject target;
 [HideInInspector]
 [NonSerialized]
 public Vector3 startPosition;
 [HideInInspector]
 [NonSerialized]
	public Vector3 targetPosition;
	
	private float distance;
	private float startTime;
	
	private GameManagerBehavior gameManager;

	// Use this for initialization
	void Start () {
		startTime = Time.time;
		distance = Vector3.Distance (startPosition, targetPosition);
		GameObject gm = GameObject.Find("GameManager");
		gameManager = gm.GetComponent<GameManagerBehavior>();	
	}
	
	// Update is called once per frame
	void Update () {
		// 1 
		float timeInterval = Time.time - startTime;
		gameObject.transform.position = Vector3.Lerp(startPosition, targetPosition, timeInterval * speed / distance);
		
		// 2 
		if (gameObject.transform.position.Equals(targetPosition)) {
			if (target != null) {
				// 3
				Transform healthBarTransform = target.transform.FindChild("HealthBar");
				HealthBar healthBar = 
					healthBarTransform.gameObject.GetComponent<HealthBar>();
				healthBar.currentHealth -= Mathf.Max(damage, 0);
				// 4
				if (healthBar.currentHealth <= 0) {
					Destroy(target);
					AudioSource audioSource = target.GetComponent<AudioSource>();
					AudioSource.PlayClipAtPoint(audioSource.clip, transform.position);
					
					gameManager.Gold += 50;
				}
			}
			Destroy(gameObject);
		}	
	}

  private int damage {
    get {
      return bulletConfig.damage;
    }
  }

  private float speed {
    get {
      return bulletConfig.speed;
    }
  }
}
